/* $Id: point.cc,v 1.7 2005/12/26 22:29:22 jonathan Exp $
 * Jonathan Ledlie, Harvard University.
 * Copyright 2005.  All rights reserved.
 */

#include "vivaldi.h"
#include "point.h"

const double MIN_HEIGHT = 0.001;

Point::Point (int s) {
  stamp = s;
}

Point::Point () {
  v = new double[DIMENSIONS];
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = INITIAL_VECTOR_VALUE;
  }
  stamp = 0;
  height = INITIAL_VECTOR_VALUE;
}

Point::Point (const double *vec, double h) {
  v = new double[DIMENSIONS];
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = vec[i];
  }
  stamp = 0;
  height = h;
}


void Point::clear () {
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = INITIAL_VECTOR_VALUE;
  }
  stamp = 0;
  height = INITIAL_VECTOR_VALUE;
}

Point::Point (const Point *vec) {
  v = new double[DIMENSIONS];
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = vec->v[i];
  }
  stamp = vec->stamp;
  height = vec->height;
}

Point::~Point () {
  delete [] v;
}

bool Point::operator<(const Point *rhs) const {
  return stamp < rhs->stamp;
}

Point* Point::getDirection (const Point *p) {
  double coordDistance = getEuclideanDistance(p);
  if (coordDistance == 0)
    return NULL;
  Point* unitVector = new Point();
  for (int i = 0; i < DIMENSIONS; i++) {
    unitVector->v[i] = (v[i] - p->v[i]) / coordDistance;	    
  }
  unitVector->height = height + p->height;
  return unitVector;
}

double Point::getEuclideanDistance (const Point *b) {
  return getPlanarDistance(b) + height + b->height;
}

double Point::getAbsoluteDistance (const Point *b) {
  return getPlanarDistance(b) + fabs(height - b->height);
}

double Point::getPlanarDistance (const Point *b) {
  double sum = 0.;
  for (int i = 0; i < DIMENSIONS; i++) {
    sum += pow (v[i]-b->v[i],2.);
  }
  if (sum == 0.) return 0.;
  double dist = sqrt (sum);
  if (isnan(dist)) {
    cout << "sum " << sum << this << endl;
  }
  assert (!isnan(dist));
  return dist;
}

double Point::length () {
  double sum = 0;
  for (int i = 0; i < DIMENSIONS; i++) {
    sum += pow (v[i],2.);
  }
  return sqrt (sum) + height;
}

void Point::bump () {
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = randPct();
  }
  if (USE_HEIGHT)
    height = randPct();
}

void Point::assign (const Point *vec) {
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = vec->v[i];
  }
  stamp = vec->stamp;
  height = vec->height;
}


bool Point::isInInitialState () {
  for (int i = 0; i < DIMENSIONS; i++) {
    if (v[i] != INITIAL_VECTOR_VALUE)
      return false;
  }
  if (height != INITIAL_VECTOR_VALUE)
    return false;
  return true;
}

void Point::scale (double s) {
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = v[i] * s;
  }
  height *= s;
}

void Point::add (const Point *adjustment) {
  for (int i = 0; i < DIMENSIONS; i++) {
    v[i] = v[i] + adjustment->v[i];
  }
  height += adjustment->height;
}

void Point::checkHeight () {
  if (USE_HEIGHT && height < MIN_HEIGHT) {
    height = MIN_HEIGHT;
  }
}

Point* Point::getRandomUnitVector () {
  Point* unitVector = new Point();
  double length = 0.;
  for (int i = 0; i < DIMENSIONS; i++) {
    unitVector->v[i] = random();
    length += unitVector->v[i] * unitVector->v[i];
  }  
  length = sqrt(length);

  for (int i = 0; i < DIMENSIONS; i++) {
    unitVector->v[i] /= length;
  }	
  unitVector->height = 0;
  if (USE_HEIGHT) {
    unitVector->height = randPct();
    printf ("random height %f\n", unitVector->height);
  }

  return unitVector;
}



ostream& operator << (ostream& os, Point *p) 
{
  os << "[";
  for (int i = 0; i < DIMENSIONS; i++) {
    os << p->v[i];
    if (i < DIMENSIONS-1)
      os << " ";
  }
  if (USE_HEIGHT)
    os << " h " << p->height;
  os << "]";
  return os;
}

void Point::print () {
  for (int i = 0; i < DIMENSIONS; i++) {
    printf ("%5.3f", v[i]);
    if (i < DIMENSIONS-1)
      printf (" ");
  }
  if (USE_HEIGHT)
    printf (" h %f", height);

}
